@@ -2,10 +2,10 @@
 Locking mechanism to ensure no two compilations occur simultaneously
 in the same compilation directory (which can cause crashes).
 """
+
 import os
 import threading
 from contextlib import contextmanager
-from typing import Optional, Union
 
 import filelock
 
@@ -46,9 +46,9 @@ def force_unlock(lock_dir: os.PathLike):
 
 @contextmanager
 def lock_ctx(
-    lock_dir: Optional[Union[str, os.PathLike]] = None,
+    lock_dir: str | os.PathLike | None = None,
     *,
-    timeout: Optional[float] = None,
+    timeout: float | None = None,
 ):
     """Context manager that wraps around FileLock and SoftFileLock from filelock package.
 
