@@ -508,11 +508,13 @@ class Validator(Feature):
 
 
 class ReplaceValidate(History, Validator):
-    pickle_rm_attr = (
-        ["replace_validate", "replace_all_validate", "replace_all_validate_remove"]
-        + History.pickle_rm_attr
-        + Validator.pickle_rm_attr
-    )
+    pickle_rm_attr = [
+        "replace_validate",
+        "replace_all_validate",
+        "replace_all_validate_remove",
+        *History.pickle_rm_attr,
+        *Validator.pickle_rm_attr,
+    ]
 
     def on_attach(self, fgraph):
         for attr in (
