@@ -14,9 +14,10 @@ import scipy.stats
 
 from pytensor.configdefaults import config
 from pytensor.gradient import grad_not_implemented, grad_undefined
-from pytensor.scalar.basic import BinaryScalarOp, ScalarOp, UnaryScalarOp
-from pytensor.scalar.basic import abs as scalar_abs
 from pytensor.scalar.basic import (
+    BinaryScalarOp,
+    ScalarOp,
+    UnaryScalarOp,
     as_scalar,
     complex_types,
     constant,
@@ -42,6 +43,7 @@ from pytensor.scalar.basic import (
     upgrade_to_float64,
     upgrade_to_float_no_complex,
 )
+from pytensor.scalar.basic import abs as scalar_abs
 from pytensor.scalar.loop import ScalarLoop
 
 
@@ -152,8 +154,9 @@ class Erfcx(UnaryScalarOp):
 
     def c_header_dirs(self, **kwargs):
         # Using the Faddeeva.hh (c++) header for Faddeevva.cc
-        res = super().c_header_dirs(**kwargs) + [
-            os.path.join(os.path.dirname(__file__), "c_code")
+        res = [
+            *super().c_header_dirs(**kwargs),
+            os.path.join(os.path.dirname(__file__), "c_code"),
         ]
         return res
 
@@ -618,11 +621,8 @@ class Chi2SF(BinaryScalarOp):
         (z,) = out
         if node.inputs[0].type in float_types:
             dtype = "npy_" + node.outputs[0].dtype
-            return (
-                """%(z)s =
-                (%(dtype)s) 1 - GammaP(%(k)s/2., %(x)s/2.);"""
-                % locals()
-            )
+            return """{z} =
+                ({dtype}) 1 - GammaP({k}/2., {x}/2.);""".format(**locals())
         raise NotImplementedError("only floatingpoint is implemented")
 
     def __eq__(self, other):
@@ -631,6 +631,13 @@ class Chi2SF(BinaryScalarOp):
     def __hash__(self):
         return hash(type(self))
 
+    def c_code_cache_version(self):
+        v = super().c_code_cache_version()
+        if v:
+            return (2, *v)
+        else:
+            return v
+
 
 chi2sf = Chi2SF(upgrade_to_float64, name="chi2sf")
 
@@ -667,11 +674,8 @@ class GammaInc(BinaryScalarOp):
         (z,) = out
         if node.inputs[0].type in float_types:
             dtype = "npy_" + node.outputs[0].dtype
-            return (
-                """%(z)s =
-                (%(dtype)s) GammaP(%(k)s, %(x)s);"""
-                % locals()
-            )
+            return """{z} =
+                ({dtype}) GammaP({k}, {x});""".format(**locals())
         raise NotImplementedError("only floatingpoint is implemented")
 
     def __eq__(self, other):
@@ -680,6 +684,13 @@ class GammaInc(BinaryScalarOp):
     def __hash__(self):
         return hash(type(self))
 
+    def c_code_cache_version(self):
+        v = super().c_code_cache_version()
+        if v:
+            return (2, *v)
+        else:
+            return v
+
 
 gammainc = GammaInc(upgrade_to_float, name="gammainc")
 
@@ -716,11 +727,8 @@ class GammaIncC(BinaryScalarOp):
         (z,) = out
         if node.inputs[0].type in float_types:
             dtype = "npy_" + node.outputs[0].dtype
-            return (
-                """%(z)s =
-                (%(dtype)s) GammaQ(%(k)s, %(x)s);"""
-                % locals()
-            )
+            return """{z} =
+                ({dtype}) GammaQ({k}, {x});""".format(**locals())
         raise NotImplementedError("only floatingpoint is implemented")
 
     def __eq__(self, other):
@@ -729,6 +737,13 @@ class GammaIncC(BinaryScalarOp):
     def __hash__(self):
         return hash(type(self))
 
+    def c_code_cache_version(self):
+        v = super().c_code_cache_version()
+        if v:
+            return (2, *v)
+        else:
+            return v
+
 
 gammaincc = GammaIncC(upgrade_to_float, name="gammaincc")
 
@@ -1025,11 +1040,8 @@ class GammaU(BinaryScalarOp):
         (z,) = out
         if node.inputs[0].type in float_types:
             dtype = "npy_" + node.outputs[0].dtype
-            return (
-                """%(z)s =
-                (%(dtype)s) upperGamma(%(k)s, %(x)s);"""
-                % locals()
-            )
+            return """{z} =
+                ({dtype}) upperGamma({k}, {x});""".format(**locals())
         raise NotImplementedError("only floatingpoint is implemented")
 
     def __eq__(self, other):
@@ -1066,11 +1078,8 @@ class GammaL(BinaryScalarOp):
         (z,) = out
         if node.inputs[0].type in float_types:
             dtype = "npy_" + node.outputs[0].dtype
-            return (
-                """%(z)s =
-                (%(dtype)s) lowerGamma(%(k)s, %(x)s);"""
-                % locals()
-            )
+            return """{z} =
+                ({dtype}) lowerGamma({k}, {x});""".format(**locals())
         raise NotImplementedError("only floatingpoint is implemented")
 
     def __eq__(self, other):
@@ -1321,7 +1330,7 @@ class Sigmoid(UnaryScalarOp):
     def c_code_cache_version(self):
         v = super().c_code_cache_version()
         if v:
-            return (2,) + v
+            return (2, *v)
         else:
             return v
 
@@ -1403,7 +1412,7 @@ class Softplus(UnaryScalarOp):
     def c_code_cache_version(self):
         v = super().c_code_cache_version()
         if v:
-            return (3,) + v
+            return (3, *v)
         else:
             return v
 
