@@ -30,9 +30,8 @@ from pytensor.gradient import (
 from pytensor.graph.basic import Apply, graph_inputs
 from pytensor.graph.null_type import NullType
 from pytensor.graph.op import Op
-from pytensor.tensor.math import add, dot, exp, sigmoid, sqr
+from pytensor.tensor.math import add, dot, exp, sigmoid, sqr, tanh
 from pytensor.tensor.math import sum as pt_sum
-from pytensor.tensor.math import tanh
 from pytensor.tensor.random import RandomStream
 from pytensor.tensor.type import (
     discrete_dtypes,
@@ -282,7 +281,7 @@ class TestGrad:
         o = TestGrad.Obj1()
         a1 = o.make_node()
         g0, g1, g2 = grad(
-            a1.outputs[0], a1.inputs + [scalar("z")], disconnected_inputs="ignore"
+            a1.outputs[0], [*a1.inputs, scalar("z")], disconnected_inputs="ignore"
         )
         assert o.gval0 is g0
         assert o.gval1 is g1
